#!/usr/local/bin/perl
use strict;
use warnings;

my $b_ref = read_board( <DATA> );

for ( 1 .. shift || 20 )
{
  print_board( $b_ref );
  my $new_b_ref = transform_board( $b_ref );
  $b_ref = $new_b_ref;
  sleep 1;
}

sub read_board
{
  chomp( my @lines = @_ );
  my @rows = map { [ map { /O/ ? 1 : 0 } split // ] } @lines;
  return \@rows;
}

sub transform_board
{
  my $b_ref = shift;

  my @new_board;
  for my $y ( 0 .. $#$b_ref )
  {
    my @new_row;
    for my $x ( 0 .. $#{ $b_ref->[$y] } )
    {
      push @new_row, compute_cell( $b_ref, $y, $x );
    }
    push @new_board, \@new_row;
  }
  return \@new_board;
}

sub compute_cell
{
  my ($b_ref, $y, $x) = @_;

  my $alive = $b_ref->[$y][$x];
  return $alive if $y == 0 || $x == 0 || $y == $#$b_ref || $x == $#{ $b_ref->[$y] };
  my $live_neighbors;
  for my $offset_ref ( [-1,-1], [-1,0], [-1,1],
                       [0,-1],          [0,1],
                       [1, -1], [1,0],  [1,1] )
  {
    $live_neighbors += $b_ref->[ $y+$offset_ref->[0] ][ $x+$offset_ref->[1] ];
  }
  return $alive && ($live_neighbors == 2 || $live_neighbors == 3) ? 1 :   # Live cells with 2 or 3 live neighbors survive
         $alive ? 0 :                                                     # Other live cells die from starvation or overcrowding
         $live_neighbors == 3 ? 1 :                                       # Dead cells with exactly 3 live neighbors reanimate
         0;                                                               # Otherwise they stay dead
}

sub print_board
{
  my $b_ref = shift;

  my $width;
  for ( @$b_ref )
  {
    $width ||= @$_;
    print $_ ? 'O' : '.' for @$_;
    print "\n";
  }
  print '-' x $width, "\n";
}

__END__
.................
.................
....OOO...OOO....
.................
..O....O.O....O..
..O....O.O....O..
..O....O.O....O..
....OOO...OOO....
.................
....OOO...OOO....
..O....O.O....O..
..O....O.O....O..
..O....O.O....O..
.................
....OOO...OOO....
.................
.................
